module jp.ku6.windows.directx.dxgi;

import jp.ku6.windows.win32api.windef;
import jp.ku6.windows.win32api.basetyps;
import jp.ku6.windows.win32api.unknwn;
import jp.ku6.windows.directx.dxgitype;
import jp.ku6.windows.directx.dxgiformat;

immutable uint DXGI_CPU_ACCESS_NONE             = 0;
immutable uint DXGI_CPU_ACCESS_DYNAMIC          = 1;
immutable uint DXGI_CPU_ACCESS_READ_WRITE       = 2;
immutable uint DXGI_CPU_ACCESS_SCRATCH          = 3;
immutable uint DXGI_CPU_ACCESS_FIELD            = 15;
immutable uint DXGI_USAGE_SHADER_INPUT          = 1 << (0 + 4);
immutable uint DXGI_USAGE_RENDER_TARGET_OUTPUT  = 1 << (1 + 4);
immutable uint DXGI_USAGE_BACK_BUFFER           = 1 << (2 + 4);
immutable uint DXGI_USAGE_SHARED                = 1 << (3 + 4);
immutable uint DXGI_USAGE_READ_ONLY             = 1 << (4 + 4);
immutable uint DXGI_USAGE_DISCARD_ON_PRESENT    = 1 << (5 + 4);
immutable uint DXGI_USAGE_UNORDERED_ACCESS      = 1 << (6 + 4);
alias UINT DXGI_USAGE;

struct DXGI_FRAME_STATISTICS
{
	UINT PresentCount;
	UINT PresentRefreshCount;
	UINT SyncRefreshCount;
	LARGE_INTEGER SyncQPCTime;
	LARGE_INTEGER SyncGPUTime;
}

struct DXGI_MAPPED_RECT
{
	INT Pitch;
	BYTE* pBits;
}

struct LUID
{
	DWORD LowPart;
	LONG HighPart;
}
alias LUID* PLUID;

struct DXGI_ADAPTER_DESC
{
	WCHAR[128] Description;
	UINT VendorId;
	UINT DeviceId;
	UINT SubSysId;
	UINT Revision;
	SIZE_T DedicatedVideoMemory;
	SIZE_T DedicatedSystemMemory;
	SIZE_T SharedSystemMemory;
	LUID AdapterLuid;
}

struct DXGI_OUTPUT_DESC
{
	WCHAR[32] DeviceName;
	RECT DesktopCoordinates;
	BOOL AttachedToDesktop;
	DXGI_MODE_ROTATION Rotation;
	HMONITOR Monitor;
}


struct DXGI_SHARED_RESOURCE
{
	HANDLE Handle;
}

immutable uint DXGI_RESOURCE_PRIORITY_MINIMUM  = 0x28000000;
immutable uint DXGI_RESOURCE_PRIORITY_LOW      = 0x50000000;
immutable uint DXGI_RESOURCE_PRIORITY_NORMAL   = 0x78000000;
immutable uint DXGI_RESOURCE_PRIORITY_HIGH     = 0xa0000000;
immutable uint 	DXGI_RESOURCE_PRIORITY_MAXIMUM = 0xc8000000;


enum DXGI_RESIDENCY
{
	DXGI_RESIDENCY_FULLY_RESIDENT            = 1,
	DXGI_RESIDENCY_RESIDENT_IN_SHARED_MEMORY = 2,
	DXGI_RESIDENCY_EVICTED_TO_DISK           = 3
}

struct DXGI_SURFACE_DESC
{
	UINT Width;
	UINT Height;
	DXGI_FORMAT Format;
	DXGI_SAMPLE_DESC SampleDesc;
}

enum DXGI_SWAP_EFFECT
{
	DXGI_SWAP_EFFECT_DISCARD    = 0,
	DXGI_SWAP_EFFECT_SEQUENTIAL = 1
}

enum DXGI_SWAP_CHAIN_FLAG
{
	DXGI_SWAP_CHAIN_FLAG_NONPREROTATED     = 1,
	DXGI_SWAP_CHAIN_FLAG_ALLOW_MODE_SWITCH = 2,
	DXGI_SWAP_CHAIN_FLAG_GDI_COMPATIBLE    = 4
}

struct DXGI_SWAP_CHAIN_DESC
{
	DXGI_MODE_DESC BufferDesc;
	DXGI_SAMPLE_DESC SampleDesc;
	DXGI_USAGE BufferUsage;
	UINT BufferCount;
	HWND OutputWindow;
	BOOL Windowed;
	DXGI_SWAP_EFFECT SwapEffect;
	UINT Flags;
}


extern (Windows)
{
	interface IDXGIObject : IUnknown
	{
		HRESULT SetPrivateData(REFGUID Name, UINT DataSize, const(void)* pData);
		HRESULT SetPrivateDataInterface(REFGUID Name, const(IUnknown) pUnknown);
		HRESULT GetPrivateData(REFGUID Name, UINT* pDataSize, void* pData);
		HRESULT GetParent(REFIID riid, void** ppParent);
	}

	interface IDXGIDeviceSubObject : IDXGIObject
	{
		HRESULT GetDevice(REFIID riid, void** ppDevice);
	}

	
	interface IDXGIResource : IDXGIDeviceSubObject
	{
		HRESULT GetSharedHandle(HANDLE* pSharedHandle);
		HRESULT GetUsage(DXGI_USAGE* pUsage);
		HRESULT SetEvictionPriority(UINT EvictionPriority);
		HRESULT GetEvictionPriority(UINT* pEvictionPriority);
	}

	interface IDXGIKeyedMutex : IDXGIDeviceSubObject
	{
		HRESULT AcquireSync(UINT64 Key, DWORD dwMilliseconds);
		HRESULT ReleaseSync(UINT64 Key);
	}
}

immutable uint DXGI_MAP_READ    = 1;
immutable uint DXGI_MAP_WRITE   = 2;
immutable uint DXGI_MAP_DISCARD = 4;

extern (Windows)
{
	interface IDXGISurface : IDXGIDeviceSubObject
	{
		HRESULT GetDesc(DXGI_SURFACE_DESC* pDesc);
		HRESULT Map(DXGI_MAPPED_RECT* pLockedRect, UINT MapFlags);
		HRESULT Unmap();
	}

	interface IDXGISurface1 : IDXGISurface
	{
		HRESULT GetDC(BOOL Discard, HDC* phdc);
		HRESULT ReleaseDC(RECT* pDirtyRect);
	}

	interface IDXGIAdapter : IDXGIObject
	{
		HRESULT EnumOutputs(UINT Output, IDXGIOutput* ppOutput);
		HRESULT GetDesc(DXGI_ADAPTER_DESC* pDesc);
		HRESULT CheckInterfaceSupport(REFGUID InterfaceName, LARGE_INTEGER* pUMDVersion);
	}
}

immutable uint DXGI_ENUM_MODES_INTERLACED = 1;
immutable uint DXGI_ENUM_MODES_SCALING    = 2;

extern (Windows)
{
	interface IDXGIOutput : IDXGIObject
	{
		HRESULT GetDesc(DXGI_OUTPUT_DESC* pDesc);
		HRESULT GetDisplayModeList(DXGI_FORMAT EnumFormat, UINT Flags, UINT* pNumModes, DXGI_MODE_DESC* pDesc);
		HRESULT FindClosestMatchingMode(const(DXGI_MODE_DESC)* pModeToMatch, DXGI_MODE_DESC* pClosestMatch, IUnknown pConcernedDevice);
		HRESULT WaitForVBlank();
		HRESULT TakeOwnership(IUnknown pDevice, BOOL Exclusive);
		void ReleaseOwnership();
		HRESULT GetGammaControlCapabilities(DXGI_GAMMA_CONTROL_CAPABILITIES* pGammaCaps);
		HRESULT SetGammaControl(const(DXGI_GAMMA_CONTROL)* pArray);
		HRESULT GetGammaControl(DXGI_GAMMA_CONTROL* pArray);
		HRESULT SetDisplaySurface(IDXGISurface pScanoutSurface);
		HRESULT GetDisplaySurfaceData(IDXGISurface pDestination);
		HRESULT GetFrameStatistics(DXGI_FRAME_STATISTICS* pStats);
	}
}

immutable uint DXGI_MAX_SWAP_CHAIN_BUFFERS = 16;

immutable uint DXGI_PRESENT_TEST            = 0x00000001;
immutable uint DXGI_PRESENT_DO_NOT_SEQUENCE = 0x00000002;
immutable uint DXGI_PRESENT_RESTART         = 0x00000004;

extern (Windows)
{
	interface IDXGISwapChain : IDXGIDeviceSubObject
	{
		HRESULT Present(UINT SyncInterval, UINT Flags);
		HRESULT GetBuffer(UINT Buffer, REFIID riid, void** ppSurface);
		HRESULT SetFullscreenState(BOOL Fullscreen, IDXGIOutput pTarget);
		HRESULT GetFullscreenState(BOOL* pFullscreen, IDXGIOutput* ppTarget);
		HRESULT GetDesc(DXGI_SWAP_CHAIN_DESC* pDesc);
		HRESULT ResizeBuffers(UINT BufferCount, UINT Width, UINT Height, DXGI_FORMAT NewFormat, UINT SwapChainFlags);
		HRESULT ResizeTarget(const(DXGI_MODE_DESC)* pNewTargetParameters);
		HRESULT GetContainingOutput(IDXGIOutput* ppOutput);
		HRESULT GetFrameStatistics(DXGI_FRAME_STATISTICS* pStats);
		HRESULT GetLastPresentCount(UINT* pLastPresentCount);
	}
}

immutable uint DXGI_MWA_NO_WINDOW_CHANGES = 1 << 0;
immutable uint DXGI_MWA_NO_ALT_ENTER      = 1 << 1;
immutable uint DXGI_MWA_NO_PRINT_SCREEN   = 1 << 2;
immutable uint DXGI_MWA_VALID             = 0x7;

extern (Windows)
{
	interface IDXGIFactory : IDXGIObject
	{
		HRESULT EnumAdapters(UINT Adapter, IDXGIAdapter* ppAdapter);
		HRESULT MakeWindowAssociation(HWND WindowHandle, UINT Flags);
		HRESULT GetWindowAssociation(HWND* pWindowHandle);
		HRESULT CreateSwapChain(IUnknown pDevice, DXGI_SWAP_CHAIN_DESC* pDesc, IDXGISwapChain* ppSwapChain);
		HRESULT CreateSoftwareAdapter(HMODULE Module, IDXGIAdapter* ppAdapter);
	}

	alias HRESULT function(REFIID riid, void** ppFactory) CreateDXGIFactoryFunction;
	alias HRESULT function(REFIID riid, void** ppFactory) CreateDXGIFactory1Function;

	interface IDXGIDevice : IDXGIObject
	{
		HRESULT GetAdapter(IDXGIAdapter* pAdapter);
		HRESULT CreateSurface(const(DXGI_SURFACE_DESC)* pDesc, UINT NumSurfaces, DXGI_USAGE Usage, const(DXGI_SHARED_RESOURCE)* pSharedResource, IDXGISurface* ppSurface);
		HRESULT QueryResourceResidency(const(IUnknown*) ppResources, DXGI_RESIDENCY* pResidencyStatus, UINT NumResources);
		HRESULT SetGPUThreadPriority(INT Priority);
		HRESULT GetGPUThreadPriority(INT* pPriority);
	}
}

enum DXGI_ADAPTER_FLAG
{
	DXGI_ADAPTER_FLAG_NONE        = 0,
	DXGI_ADAPTER_FLAG_REMOTE      = 1,
	DXGI_ADAPTER_FLAG_FORCE_DWORD = 0xffffffff
}

struct DXGI_ADAPTER_DESC1
{
	WCHAR[128] Description;
	UINT VendorId;
	UINT DeviceId;
	UINT SubSysId;
	UINT Revision;
	SIZE_T DedicatedVideoMemory;
	SIZE_T DedicatedSystemMemory;
	SIZE_T SharedSystemMemory;
	LUID AdapterLuid;
	UINT Flags;
}

struct DXGI_DISPLAY_COLOR_SPACE
{
	FLOAT[8][2] PrimaryCoordinates;
	FLOAT[16][2] WhitePoints;
}

extern (Windows)
{
	interface IDXGIFactory1 : IDXGIFactory
	{
		HRESULT EnumAdapters1(UINT Adapter, IDXGIAdapter1* ppAdapter);
		BOOL IsCurrent();
	}

	interface IDXGIAdapter1 : IDXGIAdapter
	{
		HRESULT GetDesc1(DXGI_ADAPTER_DESC1* pDesc);
	}

	interface IDXGIDevice1 : IDXGIDevice
	{
		HRESULT SetMaximumFrameLatency(UINT MaxLatency);
		HRESULT GetMaximumFrameLatency(UINT* pMaxLatency);
	}
}

immutable GUID IID_IDXGIObject          = {0xaec22fb8, 0x76f3, 0x4639, [0x9b, 0xe0, 0x28, 0xeb, 0x43, 0xa6, 0x7a, 0x2e]};
immutable GUID IID_IDXGIDeviceSubObject = {0x3d3e0379, 0xf9de, 0x4d58, [0xbb, 0x6c, 0x18, 0xd6, 0x29, 0x92, 0xf1, 0xa6]};
immutable GUID IID_IDXGIResource        = {0x035f3ab4, 0x482e, 0x4e50, [0xb4, 0x1f, 0x8a, 0x7f, 0x8b, 0xd8, 0x96, 0x0b]};
immutable GUID IID_IDXGIKeyedMutex      = {0x9d8e1289, 0xd7b3, 0x465f, [0x81, 0x26, 0x25, 0x0e, 0x34, 0x9a, 0xf8, 0x5d]};
immutable GUID IID_IDXGISurface         = {0xcafcb56c, 0x6ac3, 0x4889, [0xbf, 0x47, 0x9e, 0x23, 0xbb, 0xd2, 0x60, 0xec]};
immutable GUID IID_IDXGISurface1        = {0x4AE63092, 0x6327, 0x4c1b, [0x80, 0xAE, 0xBF, 0xE1, 0x2E, 0xA3, 0x2B, 0x86]};
immutable GUID IID_IDXGIAdapter         = {0x2411e7e1, 0x12ac, 0x4ccf, [0xbd, 0x14, 0x97, 0x98, 0xe8, 0x53, 0x4d, 0xc0]};
immutable GUID IID_IDXGIOutput          = {0xae02eedb, 0xc735, 0x4690, [0x8d, 0x52, 0x5a, 0x8d, 0xc2, 0x02, 0x13, 0xaa]};
immutable GUID IID_IDXGISwapChain       = {0x310d36a0, 0xd2e7, 0x4c0a, [0xaa, 0x04, 0x6a, 0x9d, 0x23, 0xb8, 0x88, 0x6a]};
immutable GUID IID_IDXGIFactory         = {0x7b7166ec, 0x21c7, 0x44ae, [0xb2, 0x1a, 0xc9, 0xae, 0x32, 0x1a, 0xe3, 0x69]};
immutable GUID IID_IDXGIDevice          = {0x54ec77fa, 0x1377, 0x44e6, [0x8c, 0x32, 0x88, 0xfd, 0x5f, 0x44, 0xc8, 0x4c]};
immutable GUID IID_IDXGIFactory1        = {0x770aae78, 0xf26f, 0x4dba, [0xa8, 0x29, 0x25, 0x3c, 0x83, 0xd1, 0xb3, 0x87]};
immutable GUID IID_IDXGIAdapter1        = {0x29038f61, 0x3839, 0x4626, [0x91, 0xfd, 0x08, 0x68, 0x79, 0x01, 0x1a, 0x05]};
immutable GUID IID_IDXGIDevice1         = {0x77db970f, 0x6276, 0x48ba, [0xba, 0x28, 0x07, 0x01, 0x43, 0xb4, 0x39, 0x2c]};


